﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;

namespace SaveRecordSetApp
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private static readonly string ConnectionStr = @"Server=.\SQL2008;Database=SqlSat62; Trusted_Connection=yes;";

        private int GetPacketSize()
        {
            int result;
            if (!Int32.TryParse(tbPacketSize.Text, out result))
            {
                MessageBox.Show("Invalid packet size. Using 5000", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return 5000;
            }
            return result;
        }

        private SqlConnection GetConnection()
        {
            SqlConnection conn = new SqlConnection(ConnectionStr);
            conn.Open();
            return conn;
        }

        private void TruncateTable()
        {
            using (SqlConnection conn = GetConnection())
            {
                SqlCommand cmd = new SqlCommand("truncate table dbo.DataRecords", conn);
                cmd.ExecuteNonQuery();
            }
        }

        private void btnSeparateInserts_Click(object sender, EventArgs e)
        {
            TimeSpan workTime;
            int packetSize = GetPacketSize();

            TruncateTable();
            using (SqlConnection conn = GetConnection())
            {
                DateTime startTime = DateTime.Now;
                SqlCommand insertCmd = new SqlCommand(
                    @"insert into dbo.DataRecords(ID,Field1,Field2,Field3,Field4,Field5,
                        Field6,Field7,Field8,Field9,Field10,Field11,Field12,Field13,
	                    Field14,Field15,Field16,Field17,Field18,Field19,Field20)
                    values(@ID,@Field1,@Field2,@Field3,@Field4,@Field5
                    ,@Field6,@Field7,@Field8,@Field9,@Field10,@Field11,@Field12,@Field13
                    ,@Field14,@Field15,@Field16,@Field17,@Field18,@Field19,@Field20)",conn);
                insertCmd.Parameters.Add("@ID", SqlDbType.Int);
                for (int i = 1; i <= 20; i++)
                    insertCmd.Parameters.Add("@Field" + i.ToString(), SqlDbType.VarChar, 20);
                using (SqlTransaction tran = conn.BeginTransaction(IsolationLevel.ReadCommitted))
                {
                    try
                    {
                        insertCmd.Transaction = tran;

                        for (int i = 0; i < packetSize; i++)
                        {
                            insertCmd.Parameters[0].Value = i;
                            for (int p = 1; p <= 20; p++)
                                insertCmd.Parameters[p].Value = "Parameter: " + p.ToString();
                            insertCmd.ExecuteNonQuery();
                        }
                        tran.Commit();
                    }
                    catch (Exception ex)
                    {
                        tran.Rollback();
                        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                workTime = DateTime.Now - startTime;
            }
            MessageBox.Show(workTime.TotalMilliseconds.ToString(), "Total execution time", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }










































        private void btnTvp_Click(object sender, EventArgs e)
        {
            TimeSpan workTime1, workTime2;
            int packetSize = GetPacketSize();

            TruncateTable();
            using (SqlConnection conn = GetConnection())
            {
                DateTime startTime = DateTime.Now;
                DataTable table = new DataTable();
                table.Columns.Add("ID", typeof(Int32));
                for (int i = 1; i <= 20; i++)
                    table.Columns.Add("Field" + i.ToString(), typeof(string));
                for (int i = 0; i < packetSize; i++)
                    table.Rows.Add(i, "Parameter: 1", "Parameter: 2", "Parameter: 3",
                        "Parameter: 4", "Parameter: 5", "Parameter: 6", "Parameter: 7",
                        "Parameter: 8", "Parameter: 9", "Parameter: 10", "Parameter: 11",
                        "Parameter: 12", "Parameter: 13", "Parameter: 14", "Parameter: 15",
                        "Parameter: 16", "Parameter: 17", "Parameter: 18", "Parameter: 19",
                        "Parameter: 20");
                workTime1 = DateTime.Now - startTime;
                SqlCommand insertCmd = new SqlCommand("dbo.InsertDataRecordsTVP", conn);
                insertCmd.Parameters.Add("@Data", SqlDbType.Structured);
                insertCmd.Parameters[0].TypeName = "dbo.DataRecordsTVP";
                insertCmd.Parameters[0].Value = table;
				insertCmd.ExecuteNonQuery();
                workTime2 = DateTime.Now - startTime;
            }
            MessageBox.Show(workTime1.TotalMilliseconds.ToString() + "; " + workTime2.TotalMilliseconds.ToString(), "Total execution time", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

		
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        private string PrepareElementCentricXml(int packetSize)
		{
			StringBuilder sb = new StringBuilder("<Recs>");
			for (int i = 0; i < packetSize; i++)
				sb.AppendFormat(
@"<R><ID>{0}</ID><F1>{1}</F1><F2>{2}</F2><F3>{3}</F3><F4>{4}</F4><F5>{5}</F5><F6>{6}</F6><F7>{7}</F7><F8>{8}</F8><F9>{9}</F9><F10>{10}</F10>" +
@"<F11>{11}</F11><F12>{12}</F12><F13>{13}</F13><F14>{14}</F14><F15>{15}</F15><F16>{16}</F16><F17>{17}</F17><F18>{18}</F18><F19>{19}</F19><F20>{20}</F20></R>",
i, "Parameter: 1", "Parameter: 2", "Parameter: 3","Parameter: 4", "Parameter: 5", "Parameter: 6", "Parameter: 7","Parameter: 8", "Parameter: 9", "Parameter: 10", 
"Parameter: 11","Parameter: 12", "Parameter: 13", "Parameter: 14", "Parameter: 15","Parameter: 16", "Parameter: 17", "Parameter: 18", "Parameter: 19",
					"Parameter: 20");
			sb.Append("</Recs>");
			return sb.ToString();
		}

        
        private void btnElementCentric_Click(object sender, EventArgs e)
		{
			TimeSpan workTime1, workTime2;
			int packetSize = GetPacketSize();

			TruncateTable();
			DateTime startTime = DateTime.Now;
			string xml = PrepareElementCentricXml(packetSize);
			workTime1 = DateTime.Now - startTime;
			using (SqlConnection conn = GetConnection())
			{
				SqlCommand insertCmd = new SqlCommand("dbo.InsertDataRecordsElementsXml", conn);
				insertCmd.CommandType = CommandType.StoredProcedure;
				insertCmd.Parameters.Add("@Data", SqlDbType.Xml);
				insertCmd.Parameters[0].Value = xml;
                insertCmd.CommandTimeout = 0;
				insertCmd.ExecuteNonQuery();
				workTime2 = DateTime.Now - startTime;
			}
			MessageBox.Show(workTime1.TotalMilliseconds.ToString() + "; " + workTime2.TotalMilliseconds.ToString(), "Total execution time", MessageBoxButtons.OK, MessageBoxIcon.Information);

		}





































        private string PrepareAttributeCentricXml(int packetSize)
        {
            StringBuilder sb = new StringBuilder("<Recs>");
            for (int i = 0; i < packetSize; i++)
                sb.AppendFormat(
"<R ID=\"{0}\" F1=\"{1}\" F2=\"{2}\" F3=\"{3}\" F4=\"{4}\" F5=\"{5}\" F6=\"{6}\" F7=\"{7}\" F8=\"{8}\" F9=\"{9}\" F10=\"{10}\" " +
"F11=\"{11}\" F12=\"{12}\" F13=\"{13}\" F14=\"{14}\" F15=\"{15}\" F16=\"{16}\" F17=\"{17}\" F18=\"{18}\" F19=\"{19}\" F20=\"{20}\" />",
i, "Parameter: 1", "Parameter: 2", "Parameter: 3", "Parameter: 4", "Parameter: 5", "Parameter: 6", "Parameter: 7", "Parameter: 8", "Parameter: 9", "Parameter: 10",
"Parameter: 11", "Parameter: 12", "Parameter: 13", "Parameter: 14", "Parameter: 15", "Parameter: 16", "Parameter: 17", "Parameter: 18", "Parameter: 19",
                    "Parameter: 20");
            sb.Append("</Recs>");
            return sb.ToString();
        }
        
        private void btnAttributeCentric_Click(object sender, EventArgs e)
		{
			TimeSpan workTime1, workTime2;
			int packetSize = GetPacketSize();

			TruncateTable();
			DateTime startTime = DateTime.Now;
			string xml = PrepareAttributeCentricXml(packetSize);
			workTime1 = DateTime.Now - startTime;
			using (SqlConnection conn = GetConnection())
			{
				SqlCommand insertCmd = new SqlCommand(
                       (sender == btnAttributeCentric) ? "dbo.InsertDataRecordsAttrXml" : "dbo.InsertDataRecordsAttrXml2", conn);
				insertCmd.CommandType = CommandType.StoredProcedure;
				insertCmd.Parameters.Add("@Data", SqlDbType.Xml);
				insertCmd.Parameters[0].Value = xml;
                insertCmd.CommandTimeout = 0;
                insertCmd.ExecuteNonQuery();
				workTime2 = DateTime.Now - startTime;
			}
			MessageBox.Show(workTime1.TotalMilliseconds.ToString() + "; " + workTime2.TotalMilliseconds.ToString(), "Total execution time", MessageBoxButtons.OK, MessageBoxIcon.Information);
		}


        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        private void btnOpenXml_Click(object sender, EventArgs e)
		{
			TimeSpan workTime1, workTime2;
			int packetSize = GetPacketSize();

			TruncateTable();
			DateTime startTime = DateTime.Now;
			string xml = PrepareAttributeCentricXml(packetSize);
			workTime1 = DateTime.Now - startTime;
			using (SqlConnection conn = GetConnection())
			{
				SqlCommand insertCmd = new SqlCommand("dbo.InsertDataRecordsOpenXML", conn);
				insertCmd.CommandType = CommandType.StoredProcedure;
				insertCmd.Parameters.Add("@Data", SqlDbType.Xml);
				insertCmd.Parameters[0].Value = xml;
				insertCmd.ExecuteNonQuery();
				workTime2 = DateTime.Now - startTime;
			}
			MessageBox.Show(workTime1.TotalMilliseconds.ToString() + "; " + workTime2.TotalMilliseconds.ToString(), "Total execution time", MessageBoxButtons.OK, MessageBoxIcon.Information);

		}

    }
}
